// convert lines from a Diagram document into sinusoids.
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Wed Jun 25 15:33:07 GMT-0800 1997
// Last Modified: Mon Jun 30 10:38:36 GMT-0800 1997
// Filename:      .../sig/examples/line2sine/OscParameter.cc
// Syntax:        C++
// $Smake:        cc -Wall -g -c %b.cc -I../../include && rm -f %b.o
//

#include <string.h>
#include "OscParameter.h"


//////////////////////////////
//
// OscParameter::OscParameter
//

char OscParameter::defaultAmpEnv[10000] = "(0 0; 1000 1; s; 2000 0)";

OscParameter::OscParameter(void) {
   setAmp(1);
   setStart(0);
   setEnd(0);
   freqString = NULL;
   ampString = NULL;
}

OscParameter::OscParameter(OscParameter& another) {
   freqString = NULL;
   ampString = NULL;
   setAmp(another.getAmp());
   setStart(another.getStart());
   setEnd(another.getEnd());
   setFreqEnv(another.getFreqEnv());
   setAmpEnv(another.getAmpEnv());
};


//////////////////////////////
//
// OscParameter::~OscParameter
//

OscParameter::~OscParameter() {
   if (freqString != NULL) delete [] freqString;
   if (ampString != NULL) delete [] ampString;
}


//////////////////////////////
//
// OscParameter::getAmp
//

double OscParameter::getAmp(void) {
   return amp;
}



//////////////////////////////
//
// OscParameter::getAmpEnv
//

const char* OscParameter::getAmpEnv(void) {
   if (ampString == NULL) {
      return defaultAmpEnv;
   } else {
      return ampString;
   }
}



//////////////////////////////
//
// OscParameter::getFreqEnv
//

const char* OscParameter::getFreqEnv(void) {
   if (freqString == NULL) {
      cerr << "Error: no frequency envelope exists." << endl;
      exit(1);
   }
   return freqString;
}



//////////////////////////////
//
// OscParameter::getStart
//

long OscParameter::getStart(void) {
   return start;
}



//////////////////////////////
//
// OscParameter::getEnd
//

long OscParameter::getEnd(void) {
   return end;
}



//////////////////////////////
//
// OscParameter::getDuration
//

long OscParameter::getDuration(void) {
   return end - start;
}



//////////////////////////////
//
// OscParameter::operator=
//

OscParameter& OscParameter::operator=(OscParameter& another) {
   if (this == &another) return *this; // nothing to do when self
   setAmp(another.getAmp());
   setStart(another.getStart());
   setEnd(another.getEnd());
   setFreqEnv(another.getFreqEnv());
   setAmpEnv(another.getAmpEnv());
   return *this;
};



//////////////////////////////
//
// OscParameter::print
//

void OscParameter::print(void) {
   cout << "\tFrequency Envelope: " << getFreqEnv() << endl;
   cout << "\tAmplitude Envelope: " << getAmpEnv() << endl;
   cout << "\tAmplitude Scaling : " << getAmp() << endl;
   cout << "\tStart, End Sample : " << getStart() << ", " << getEnd() << endl;
}



//////////////////////////////
//
// OscParameter::setAmp
//

void OscParameter::setAmp(double anAmp) {
   amp = anAmp;
}



//////////////////////////////
//
// OscParameter::setAmpEnv
//

void OscParameter::setAmpEnv(const char* aString) {
   if (ampString != NULL) delete [] ampString;
   ampString = new char[strlen(aString)+1];
   strcpy(ampString, aString);
}



//////////////////////////////
//
// OscParameter::setFreqEnv
//

void OscParameter::setFreqEnv(const char* aString) {
   if (freqString != NULL) delete [] freqString;
   freqString = new char[strlen(aString)+1];
   strcpy(freqString, aString);
}



//////////////////////////////
//
// OscParameter::setStart
//

void OscParameter::setStart(long aStart) {
   start = aStart;
}



//////////////////////////////
//
// OscParameter::setEnd
//

void OscParameter::setEnd(long anEnd) {
   end = anEnd;
}


///////////////////////////////////////////////////////////////////////////


//////////////////////////////
//
// OscParameters::OscParameters 
//

OscParameters::OscParameters(void) {
   data.setGrowth(128);
   start.setGrowth(128);
   end.setGrowth(128);
   data.setSize(0);
   start.setSize(0);
   end.setSize(0);
   data.allowGrowth();
   start.allowGrowth();
   end.allowGrowth();
}


//////////////////////////////
//
// OscParameters::print 
//

void OscParameters::print(void) {
   for (int i=0; i<data.getSize(); i++) {
      cout << "Oscillator " << i << ":" << endl;
      data[i].print();
   }
}


//////////////////////////////
//
// OscParameters::sort 
//

void OscParameters::sort(void) {
    qsort(start.getBase(), start.getSize(), sizeof(doubleXY), xyComp);
    qsort(end.getBase(), end.getSize(), sizeof(doubleXY), xyComp);
}





//////////////////////////////
//
// xyComp -- compares two coordinates.  If first has a lower x value,
//	then returns a -1. If x values are the same, return 0. If the 
//	first x value is greater than the second, a 1 will be returned.
//

int xyComp(const void* c1, const void* c2) {
   if ( ((doubleXY*)c1)->x < ((doubleXY*)c2)->x ) {
      return -1;
   } else if ( ((doubleXY*)c1)->x > ((doubleXY*)c2)->x ) {
      return 1;
   } else {
      return 0;
   }
}




